@@ -66,8 +66,10 @@ from .tools import (
     find_markers,
     infer_path,
     calc_signature_score,
+    calculate_z_score,
     infer_doublets,
     mark_doublets,
+    clone_subset,
 )
 from .annotate_cluster import infer_cell_types, annotate, infer_cluster_names
 from .misc import search_genes, search_de_genes, find_outlier_clusters
