@@ -3,7 +3,7 @@ import pandas as pd
 from scipy.sparse import csr_matrix, coo_matrix, hstack
 
 from pegasusio import UnimodalData, MultimodalData
-from pegasusio import read_input, write_output, _get_fillna_dict
+from pegasusio import read_input, write_output, _fillna
 
 from pegasus import tools, misc
 
@@ -60,8 +60,28 @@ def analyze_one_modality(unidata: UnimodalData, output_name: str, is_raw: bool,
         if n_pc < kwargs["pca_n"]:
             logger.warning(f"UnimodalData {unidata.get_uid()} has either dimension ({unidata.shape[0]}, {unidata.shape[1]}) less than the specified number of PCs {kwargs['pca_n']}. Reduce the number of PCs to {n_pc}.")
 
+        if kwargs["nmf"] or (kwargs["batch_correction"] and kwargs["correction_method"] == "inmf"):
+            n_nmf = min(kwargs["nmf_n"], unidata.shape[0], unidata.shape[1])
+            if n_nmf < kwargs["nmf_n"]:
+                logger.warning(f"UnimodalData {unidata.get_uid()} has either dimension ({unidata.shape[0]}, {unidata.shape[1]}) less than the specified number of NMF components {kwargs['nmf_n']}. Reduce the number of NMF components to {n_nmf}.")
+
+        if kwargs["nmf"]:
+            if kwargs["batch_correction"] and kwargs["correction_method"] == "inmf":
+                logger.warning("NMF is skipped because integrative NMF is run instead.")
+            else:
+                tools.nmf(
+                    unidata,
+                    n_components=n_nmf,
+                    features="highly_variable_features",
+                    n_jobs=kwargs["n_jobs"],
+                    random_state=kwargs["random_state"],
+                )
+
+        # batch correction: Scanorama and iNMF
         if kwargs["batch_correction"] and kwargs["correction_method"] == "scanorama":
-            pca_key = tools.run_scanorama(unidata, n_components=n_pc, features="highly_variable_features", standardize=standardize, random_state=kwargs["random_state"])
+            dim_key = tools.run_scanorama(unidata, batch="Channel", n_components=n_pc, features="highly_variable_features", standardize=standardize, random_state=kwargs["random_state"])
+        elif kwargs["batch_correction"] and kwargs["correction_method"] == "inmf":
+            dim_key = tools.integrative_nmf(unidata, batch="Channel", n_components=n_nmf, features="highly_variable_features", lam=kwargs["inmf_lambda"], n_jobs=kwargs["n_jobs"], random_state = kwargs["random_state"])
         else:
             # PCA
             tools.pca(
@@ -72,17 +92,17 @@ def analyze_one_modality(unidata: UnimodalData, output_name: str, is_raw: bool,
                 n_jobs=kwargs["n_jobs"],
                 random_state=kwargs["random_state"],
             )
-            pca_key = "pca"
+            dim_key = "pca"
 
         # batch correction: Harmony
         if kwargs["batch_correction"] and kwargs["correction_method"] == "harmony":
-            pca_key = tools.run_harmony(unidata, rep="pca", n_jobs=kwargs["n_jobs"], n_clusters=kwargs["harmony_nclusters"], random_state = kwargs["random_state"])
+            dim_key = tools.run_harmony(unidata, batch="Channel", rep="pca", n_jobs=kwargs["n_jobs"], n_clusters=kwargs["harmony_nclusters"], random_state = kwargs["random_state"])
 
         # Find K neighbors
         tools.neighbors(
             unidata,
             K=kwargs["K"],
-            rep=pca_key,
+            rep=dim_key,
             n_jobs=kwargs["n_jobs"],
             random_state=kwargs["random_state"],
             full_speed=kwargs["full_speed"],
@@ -107,7 +127,7 @@ def analyze_one_modality(unidata: UnimodalData, output_name: str, is_raw: bool,
         tools.diffmap(
             unidata,
             n_components=kwargs["diffmap_ndc"],
-            rep=pca_key,
+            rep=dim_key,
             solver=kwargs["diffmap_solver"],
             max_t=kwargs["diffmap_maxt"],
             n_jobs=kwargs["n_jobs"],
@@ -119,7 +139,7 @@ def analyze_one_modality(unidata: UnimodalData, output_name: str, is_raw: bool,
         stat_mean, pvalue_mean, accept_rate = tools.calc_kBET(
             unidata,
             kwargs["kBET_batch"],
-            rep=pca_key,
+            rep=dim_key,
             K=kwargs["kBET_K"],
             alpha=kwargs["kBET_alpha"],
             n_jobs=kwargs["n_jobs"],
@@ -136,7 +156,7 @@ def analyze_one_modality(unidata: UnimodalData, output_name: str, is_raw: bool,
         tools.cluster(
             unidata,
             algo="spectral_louvain",
-            rep=pca_key,
+            rep=dim_key,
             resolution=kwargs["spectral_louvain_resolution"],
             rep_kmeans=kwargs["spectral_louvain_basis"],
             n_clusters=kwargs["spectral_louvain_nclusters"],
@@ -151,7 +171,7 @@ def analyze_one_modality(unidata: UnimodalData, output_name: str, is_raw: bool,
         tools.cluster(
             unidata,
             algo="spectral_leiden",
-            rep=pca_key,
+            rep=dim_key,
             resolution=kwargs["spectral_leiden_resolution"],
             rep_kmeans=kwargs["spectral_leiden_basis"],
             n_clusters=kwargs["spectral_leiden_nclusters"],
@@ -166,7 +186,7 @@ def analyze_one_modality(unidata: UnimodalData, output_name: str, is_raw: bool,
         tools.cluster(
             unidata,
             algo="louvain",
-            rep=pca_key,
+            rep=dim_key,
             resolution=kwargs["louvain_resolution"],
             random_state=kwargs["random_state"],
             class_label=kwargs["louvain_class_label"],
@@ -176,7 +196,7 @@ def analyze_one_modality(unidata: UnimodalData, output_name: str, is_raw: bool,
         tools.cluster(
             unidata,
             algo="leiden",
-            rep=pca_key,
+            rep=dim_key,
             resolution=kwargs["leiden_resolution"],
             n_iter=kwargs["leiden_niter"],
             random_state=kwargs["random_state"],
@@ -187,7 +207,7 @@ def analyze_one_modality(unidata: UnimodalData, output_name: str, is_raw: bool,
     if kwargs["net_umap"]:
         tools.net_umap(
             unidata,
-            rep=pca_key,
+            rep=dim_key,
             n_jobs=kwargs["n_jobs"],
             n_neighbors=kwargs["umap_K"],
             min_dist=kwargs["umap_min_dist"],
@@ -226,7 +246,7 @@ def analyze_one_modality(unidata: UnimodalData, output_name: str, is_raw: bool,
     if kwargs["tsne"]:
         tools.tsne(
             unidata,
-            rep=pca_key,
+            rep=dim_key,
             n_jobs=kwargs["n_jobs"],
             perplexity=kwargs["tsne_perplexity"],
             random_state=kwargs["random_state"],
@@ -236,7 +256,7 @@ def analyze_one_modality(unidata: UnimodalData, output_name: str, is_raw: bool,
     if kwargs["umap"]:
         tools.umap(
             unidata,
-            rep=pca_key,
+            rep=dim_key,
             n_neighbors=kwargs["umap_K"],
             min_dist=kwargs["umap_min_dist"],
             spread=kwargs["umap_spread"],
@@ -270,14 +290,14 @@ def analyze_one_modality(unidata: UnimodalData, output_name: str, is_raw: bool,
                 if value in unidata.obs:
                     clust_attr = value
                     break
-        
+
         if channel_attr is not None:
             logger.info(f"For doublet inference, channel_attr={channel_attr}.")
         if clust_attr is not None:
             logger.info(f"For doublet inference, clust_attr={clust_attr}.")
 
         tools.infer_doublets(unidata, channel_attr = channel_attr, clust_attr = clust_attr, expected_doublet_rate = kwargs["expected_doublet_rate"], n_jobs = kwargs["n_jobs"], random_state = kwargs["random_state"], plot_hist = output_name)
-        
+
         dbl_clusts = None
         if clust_attr is not None:
             clusts = []
@@ -331,12 +351,11 @@ def analyze_one_modality(unidata: UnimodalData, output_name: str, is_raw: bool,
 
             new_genome = unidata.get_genome()
             if new_genome != append_data.get_genome():
-                new_genome = f"{new_genome}_and_{append_data.get_genome()}" 
+                new_genome = f"{new_genome}_and_{append_data.get_genome()}"
 
             feature_metadata = pd.concat([unidata.feature_metadata, append_df], axis = 0)
             feature_metadata.reset_index(inplace = True)
-            feature_metadata.fillna(value = _get_fillna_dict(unidata.feature_metadata), inplace = True)
-
+            _fillna(feature_metadata)
             unidata = UnimodalData(unidata.barcode_metadata, feature_metadata, {"X": X, "raw.X": rawX}, unidata.uns.mapping, unidata.obsm.mapping, unidata.varm.mapping) # uns.mapping, obsm.mapping and varm.mapping are passed by reference
             unidata.uns["genome"] = new_genome
 
@@ -356,11 +375,16 @@ def analyze_one_modality(unidata: UnimodalData, output_name: str, is_raw: bool,
                 )
 
     if kwargs["output_h5ad"]:
+        import time
+        start_time = time.perf_counter()
         adata = unidata.to_anndata()
-        adata.uns["scale.data"] = adata.uns.pop("_tmp_fmat_highly_variable_features")  # assign by reference
-        adata.uns["scale.data.rownames"] = unidata.var_names[unidata.var["highly_variable_features"]].values
+        if "_tmp_fmat_highly_variable_features" in adata.uns:
+            adata.uns["scale.data"] = adata.uns.pop("_tmp_fmat_highly_variable_features")  # assign by reference
+            adata.uns["scale.data.rownames"] = unidata.var_names[unidata.var["highly_variable_features"]==True].values
         adata.write(f"{output_name}.h5ad", compression="gzip")
         del adata
+        end_time = time.perf_counter()
+        logger.info(f"H5AD file {output_name}.h5ad is written. Time spent = {end_time - start_time:.2f}s.")
 
     # write out results
     if kwargs["output_loom"]:
