@@ -44,7 +44,7 @@ from .visualization import (
 )
 from .diff_expr import de_analysis, markers, write_results_to_excel, run_de_analysis
 from .gradient_boosting import find_markers, run_find_markers
-from .subcluster_utils import get_anndata_for_subclustering
-from .signature_score import calc_signature_score
+from .subcluster_utils import clone_subset
+from .signature_score import calc_signature_score, calculate_z_score
 from .doublet_detection import infer_doublets, mark_doublets
 from .nmf import nmf, integrative_nmf
