@@ -41,12 +41,12 @@ def jump_method(
 
     rep: ``str``, optional, default: ``"pca"``
         The embedding representation used for clustering. Keyword ``'X_' + rep`` must exist in ``data.obsm``. By default, use PCA coordinates.
-    
+
     K_max: ``int``, optional, default: 40
         The maximum number of clusters to try.
 
     Y: ``float``, optional, default: ``None``
-        The transformation power used. If None, use min(data.shape[1] / 3.0, 3.0). 
+        The transformation power used. If None, use min(data.shape[1] / 3.0, 3.0).
 
     n_jobs : `int`, optional (default: -1)
         Number of threads to use. -1 refers to using all physical CPU cores.
@@ -122,7 +122,7 @@ def _find_optimal_resolution(algo, module, optimal_k, resol_max, G, random_state
             membership = membership_mid
         else:
             resol_l = resol_mid
-    
+
     if resol is None:
         resol = resol_r
         membership = _run_community_detection(algo, module, G, resol, random_state, n_iter)
@@ -153,7 +153,7 @@ def louvain(
 
     resolution: ``int``, optional, default: ``1.3``
         Resolution factor. Higher resolution tends to find more clusters with smaller sizes.
-    
+
     n_clust: ``int``, optional, default: ``None``
         This option only takes effect if 'resolution = None'. Try to find an appropriate resolution by binary search such that the total number of clusters matches 'n_clust'. The range of resolution to search is (0.01, 2.0].
 
@@ -180,13 +180,13 @@ def louvain(
         print("Need louvain! Try 'pip install louvain-github'.")
 
     rep_key = "W_" + rep
-    if rep_key not in data.uns:
+    if rep_key not in data.obsp:
         raise ValueError("Cannot find affinity matrix. Please run neighbors first!")
-    W = data.uns[rep_key]
+    W = data.obsp[rep_key]
 
     G = construct_graph(W)
     if resolution is not None:
-        membership = _run_community_detection("louvain", louvain_module, G, resolution, random_state)    
+        membership = _run_community_detection("louvain", louvain_module, G, resolution, random_state)
     else:
         assert isinstance(n_clust, int)
         resolution, membership = _find_optimal_resolution("louvain", louvain_module, n_clust, 2.0, G, random_state)
@@ -195,6 +195,7 @@ def louvain(
     labels = np.array([str(x + 1) for x in membership])
     categories = natsorted(np.unique(labels))
     data.obs[class_label] = pd.Categorical(values=labels, categories=categories)
+    data.register_attr(class_label, "cluster")
 
     n_clusters = data.obs[class_label].cat.categories.size
     logger.info(f"Louvain clustering is done. Get {n_clusters} clusters.")
@@ -253,13 +254,13 @@ def leiden(
         print("Need leidenalg! Try 'pip install leidenalg'.")
 
     rep_key = "W_" + rep
-    if rep_key not in data.uns:
+    if rep_key not in data.obsp:
         raise ValueError("Cannot find affinity matrix. Please run neighbors first!")
-    W = data.uns[rep_key]
+    W = data.obsp[rep_key]
 
     G = construct_graph(W)
     if resolution is not None:
-        membership = _run_community_detection("leiden", leidenalg, G, resolution, random_state, n_iter)        
+        membership = _run_community_detection("leiden", leidenalg, G, resolution, random_state, n_iter)
     else:
         assert isinstance(n_clust, int)
         resolution, membership = _find_optimal_resolution("leiden", leidenalg, n_clust, 2.0, G, random_state, n_iter)
@@ -268,6 +269,7 @@ def leiden(
     labels = np.array([str(x + 1) for x in membership])
     categories = natsorted(np.unique(labels))
     data.obs[class_label] = pd.Categorical(values=labels, categories=categories)
+    data.register_attr(class_label, "cluster")
 
     n_clusters = data.obs[class_label].cat.categories.size
     logger.info(f"Leiden clustering is done. Get {n_clusters} clusters.")
@@ -385,14 +387,14 @@ def spectral_louvain(
         rep_kmeans = "pca"
         if f"X_{rep_kmeans}" not in data.obsm.keys():
             raise ValueError(f"Please run {rep_kmeans} first!")
-    if f"W_{rep}" not in data.uns:
+    if f"W_{rep}" not in data.obsp:
         raise ValueError("Cannot find affinity matrix. Please run neighbors first!")
 
     labels = partition_cells_by_kmeans(
         data.obsm[f"X_{rep_kmeans}"], n_clusters, n_clusters2, n_init, n_jobs, random_state,
     )
 
-    W = data.uns[f"W_{rep}"]
+    W = data.obsp[f"W_{rep}"]
 
     G = construct_graph(W)
     partition_type = louvain_module.RBConfigurationVertexPartition
@@ -409,6 +411,7 @@ def spectral_louvain(
     labels = np.array([str(x + 1) for x in partition.membership])
     categories = natsorted(np.unique(labels))
     data.obs[class_label] = pd.Categorical(values=labels, categories=categories)
+    data.register_attr(class_label, "cluster")
 
     n_clusters = data.obs[class_label].cat.categories.size
     logger.info(f"Spectral Louvain clustering is done. Get {n_clusters} clusters.")
@@ -483,14 +486,14 @@ def spectral_leiden(
         rep_kmeans = "pca"
         if f"X_{rep_kmeans}" not in data.obsm.keys():
             raise ValueError(f"Please run {rep_kmeans} first!")
-    if f"W_{rep}" not in data.uns:
+    if f"W_{rep}" not in data.obsp:
         raise ValueError("Cannot find affinity matrix. Please run neighbors first!")
 
     labels = partition_cells_by_kmeans(
         data.obsm[f"X_{rep_kmeans}"], n_clusters, n_clusters2, n_init, n_jobs, random_state,
     )
 
-    W = data.uns[f"W_{rep}"]
+    W = data.obsp[f"W_{rep}"]
 
     G = construct_graph(W)
     partition_type = leidenalg.RBConfigurationVertexPartition
@@ -507,6 +510,7 @@ def spectral_leiden(
     labels = np.array([str(x + 1) for x in partition.membership])
     categories = natsorted(np.unique(labels))
     data.obs[class_label] = pd.Categorical(values=labels, categories=categories)
+    data.register_attr(class_label, "cluster")
 
     n_clusters = data.obs[class_label].cat.categories.size
     logger.info(f"Spectral Leiden clustering is done. Get {n_clusters} clusters.")
@@ -676,7 +680,7 @@ def split_one_cluster(
     """
     idx = np.where(data.obs[clust_label] == clust_id)[0]
     tmpdat = data[idx].copy()
-    from pegasus.tools import neighbors 
+    from pegasus.tools import neighbors
     neighbors(tmpdat, rep=rep)
     leiden(tmpdat, rep=rep, resolution=None, n_clust=n_clust, random_state=random_state)
     new_clust = data.obs[clust_label].values.astype(int)
@@ -685,4 +689,5 @@ def split_one_cluster(
         new_clust[idx[(tmpdat.obs['leiden_labels'] == label).values]] = new_label
         new_label += 1
     data.obs[res_label] = pd.Categorical(values = new_clust.astype(str), categories = np.array(range(1, new_label)).astype(str))
+    data.register_attr(res_label, "cluster")
     del tmpdat
